@@ -4,14 +4,16 @@ from typing import TYPE_CHECKING, Any, Dict, List, Optional
 
 import google.generativeai as genai
 from aiconfig.callback import CallbackEvent
-from aiconfig.default_parsers.parameterized_model_parser import ParameterizedModelParser
+from aiconfig.default_parsers.parameterized_model_parser import (
+    ParameterizedModelParser,
+)
 from aiconfig.model_parser import InferenceOptions
 from aiconfig.schema import (
     ExecuteResult,
     Output,
     OutputDataWithValue,
     Prompt,
-    PromptInput
+    PromptInput,
 )
 from aiconfig.util.config_utils import get_api_key_from_environment
 from aiconfig.util.params import resolve_prompt, resolve_prompt_string
@@ -45,7 +47,9 @@ TODO: This model Parser does not support strongly structuring the input data con
 """
 
 
-def construct_regular_outputs(response: "AsyncGenerateContentResponse") -> list[Output]:
+def construct_regular_outputs(
+    response: "AsyncGenerateContentResponse",
+) -> list[Output]:
     """
     Construct regular output per response result, without streaming enabled
     """
@@ -66,7 +70,9 @@ def construct_regular_outputs(response: "AsyncGenerateContentResponse") -> list[
     return output_list
 
 
-async def construct_stream_outputs(response: "AsyncGenerateContentResponse", options: InferenceOptions) -> list[Output]:
+async def construct_stream_outputs(
+    response: "AsyncGenerateContentResponse", options: InferenceOptions
+) -> list[Output]:
     """
     Construct Outputs while also streaming the response with stream callback
 
@@ -110,7 +116,9 @@ class GeminiModelParser(ParameterizedModelParser):
         # as an env var genai.configure() will pick up the env var
         # `GOOGLE_API_KEY`, it's just that we prefer not to call
         # `get_api_key_from_environment` multiple times if we don't need to
-        self.api_key = get_api_key_from_environment("GOOGLE_API_KEY", required=False).unwrap()
+        self.api_key = get_api_key_from_environment(
+            "GOOGLE_API_KEY", required=False
+        ).unwrap()
 
     def id(self) -> str:
         """
@@ -195,23 +203,42 @@ class GeminiModelParser(ParameterizedModelParser):
         prompts = []
 
         contents_is_str = isinstance(contents, str)
-        contents_is_list_of_strings = all(isinstance(item, str) for item in contents) if isinstance(contents, list) else False
+        contents_is_list_of_strings = (
+            all(isinstance(item, str) for item in contents)
+            if isinstance(contents, list)
+            else False
+        )
 
         # Role Dict looks like this:
         #     {'role':'user',
         #      'parts': ["Briefly explain how a computer works to a young child."]
         #     }
-        contents_is_role_dict = isinstance(contents, dict) and "role" in contents and "parts"
+        contents_is_role_dict = (
+            isinstance(contents, dict) and "role" in contents and "parts"
+        )
         # Multi Turn means that the contents is a list of dicts with alternating role and parts. See for more info: https://ai.google.dev/tutorials/python_quickstart#multi-turn_conversations
         contents_is_multi_turn = isinstance(contents, list) and all(
-            isinstance(item, dict) and "role" in item and "parts" in item for item in contents
+            isinstance(item, dict) and "role" in item and "parts" in item
+            for item in contents
         )
 
         if contents is None:
-            raise ValueError("No contents found in data. Gemini api request requires a contents field")
-        if contents_is_str or contents_is_list_of_strings or contents_is_role_dict:
+            raise ValueError(
+                "No contents found in data. Gemini api request requires a contents field"
+            )
+        if (
+            contents_is_str
+            or contents_is_list_of_strings
+            or contents_is_role_dict
+        ):
             # Just one string. Assume it's a single-turn prompt
-            prompt = Prompt(**{"name": prompt_name, "input": {"contents": contents}, "metadata": {"model": model_metadata}})
+            prompt = Prompt(
+                **{
+                    "name": prompt_name,
+                    "input": {"contents": contents},
+                    "metadata": {"model": model_metadata},
+                }
+            )
             prompts.append(prompt)
         elif contents_is_multi_turn:
             # Assume it's a multi-turn prompt. Each item in the list is a dict with role and parts
@@ -247,14 +274,23 @@ class GeminiModelParser(ParameterizedModelParser):
                 prompts.append(prompt)
                 i += 1
         else:
-            raise ValueError("Unable to parse Data into prompts. Contents data is either invalid or contains unsupported objects like protobufs.")
+            raise ValueError(
+                "Unable to parse Data into prompts. Contents data is either invalid or contains unsupported objects like protobufs."
+            )
 
-        event = CallbackEvent("on_serialize_complete", __name__, {"result": prompts})
+        event = CallbackEvent(
+            "on_serialize_complete", __name__, {"result": prompts}
+        )
         await ai_config.callback_manager.run_callbacks(event)
 
         return prompts
 
-    async def deserialize(self, prompt: Prompt, aiconfig: "AIConfigRuntime", params: Optional[Dict] = None) -> Dict:
+    async def deserialize(
+        self,
+        prompt: Prompt,
+        aiconfig: "AIConfigRuntime",
+        params: Optional[Dict] = None,
+    ) -> Dict:
         """
         Defines how to parse a prompt in the .aiconfig for a particular model
         and constructs the completion params for that model.
@@ -265,7 +301,13 @@ class GeminiModelParser(ParameterizedModelParser):
         Returns:
             dict: Model-specific completion parameters.
         """
-        await aiconfig.callback_manager.run_callbacks(CallbackEvent("on_deserialize_start", __name__, {"prompt": prompt, "params": params}))
+        await aiconfig.callback_manager.run_callbacks(
+            CallbackEvent(
+                "on_deserialize_start",
+                __name__,
+                {"prompt": prompt, "params": params},
+            )
+        )
 
         # Build Completion data
         model_settings = self.get_model_settings(prompt, aiconfig)
@@ -277,7 +319,9 @@ class GeminiModelParser(ParameterizedModelParser):
 
             resolved_prompt = resolve_prompt(prompt, params, aiconfig)
 
-            messages.append({"role": "user", "parts": [{"text": resolved_prompt}]})
+            messages.append(
+                {"role": "user", "parts": [{"text": resolved_prompt}]}
+            )
 
             completion_data["contents"] = messages
         else:
@@ -297,9 +341,19 @@ class GeminiModelParser(ParameterizedModelParser):
                     "Unable to deserialize input. Prompt input type is not a string, Gemini Model Parser expects prompt input to contain a 'contents' field as expected by Gemini API"
                 )
 
-            completion_data["contents"] = parameterize_supported_gemini_input_data(prompt_input.contents, prompt, aiconfig, params)
+            completion_data["contents"] = (
+                parameterize_supported_gemini_input_data(
+                    prompt_input.contents, prompt, aiconfig, params
+                )
+            )
 
-        await aiconfig.callback_manager.run_callbacks(CallbackEvent("on_deserialize_complete", __name__, {"output": completion_data}))
+        await aiconfig.callback_manager.run_callbacks(
+            CallbackEvent(
+                "on_deserialize_complete",
+                __name__,
+                {"output": completion_data},
+            )
+        )
         return completion_data
 
     async def run_inference(
@@ -324,12 +378,18 @@ class GeminiModelParser(ParameterizedModelParser):
             CallbackEvent(
                 "on_run_start",
                 __name__,
-                {"prompt": prompt, "options": options, "parameters": parameters},
+                {
+                    "prompt": prompt,
+                    "options": options,
+                    "parameters": parameters,
+                },
             )
         )
 
         if not self.api_key:
-            self.api_key = get_api_key_from_environment("GOOGLE_API_KEY", required=True).unwrap()
+            self.api_key = get_api_key_from_environment(
+                "GOOGLE_API_KEY", required=True
+            ).unwrap()
 
         # Gemini api stores a reference to the currently executing async event loop.
         # This causes issues on reruns for the model parser, so to alleviate this, we
@@ -356,7 +416,11 @@ class GeminiModelParser(ParameterizedModelParser):
             outputs = construct_regular_outputs(response)
 
         prompt.outputs = outputs
-        await aiconfig.callback_manager.run_callbacks(CallbackEvent("on_run_complete", __name__, {"result": prompt.outputs}))
+        await aiconfig.callback_manager.run_callbacks(
+            CallbackEvent(
+                "on_run_complete", __name__, {"result": prompt.outputs}
+            )
+        )
         return prompt.outputs
 
     def get_output_text(
@@ -389,14 +453,19 @@ https://github.com/lastmile-ai/aiconfig/blob/v1.1.8/extensions/Gemini/python/src
 """
         raise ValueError(error_message)
 
-    def _construct_chat_history(self, prompt: Prompt, aiconfig: "AIConfigRuntime", params: Dict) -> List:
+    def _construct_chat_history(
+        self, prompt: Prompt, aiconfig: "AIConfigRuntime", params: Dict
+    ) -> List:
         """
         Constructs the chat history for the model
         """
         messages = []
         # Default to always use chat context
-        remember_chat_context = not hasattr(prompt.metadata, "remember_chat_context") or (
-            hasattr(prompt.metadata, "remember_chat_context") and prompt.metadata.remember_chat_context != False
+        remember_chat_context = not hasattr(
+            prompt.metadata, "remember_chat_context"
+        ) or (
+            hasattr(prompt.metadata, "remember_chat_context")
+            and prompt.metadata.remember_chat_context != False
         )
         if remember_chat_context:
             # handle chat history. check previous prompts for the same model. if same model, add prompt and its output to completion data if it has a completed output
@@ -405,13 +474,26 @@ https://github.com/lastmile-ai/aiconfig/blob/v1.1.8/extensions/Gemini/python/src
                 if previous_prompt.name == prompt.name:
                     break
 
-                previous_prompt_is_same_model = aiconfig.get_model_name(previous_prompt) == aiconfig.get_model_name(prompt)
+                previous_prompt_is_same_model = aiconfig.get_model_name(
+                    previous_prompt
+                ) == aiconfig.get_model_name(prompt)
                 if previous_prompt_is_same_model:
-                    previous_prompt_template = resolve_prompt(previous_prompt, params, aiconfig)
-                    previous_prompt_output = aiconfig.get_latest_output(previous_prompt)
-                    previous_prompt_output_text = self.get_output_text(previous_prompt, aiconfig, previous_prompt_output)
+                    previous_prompt_template = resolve_prompt(
+                        previous_prompt, params, aiconfig
+                    )
+                    previous_prompt_output = aiconfig.get_latest_output(
+                        previous_prompt
+                    )
+                    previous_prompt_output_text = self.get_output_text(
+                        previous_prompt, aiconfig, previous_prompt_output
+                    )
 
-                    messages.append({"role": "user", "parts": [{"text": previous_prompt_template}]})
+                    messages.append(
+                        {
+                            "role": "user",
+                            "parts": [{"text": previous_prompt_template}],
+                        }
+                    )
                     messages.append(
                         {
                             "role": "model",
@@ -421,7 +503,9 @@ https://github.com/lastmile-ai/aiconfig/blob/v1.1.8/extensions/Gemini/python/src
 
         return messages
 
-    def get_prompt_template(self, prompt: Prompt, aiConfig: "AIConfigRuntime") -> str:
+    def get_prompt_template(
+        self, prompt: Prompt, aiConfig: "AIConfigRuntime"
+    ) -> str:
         """
         This method is overriden from the ParameterizedModelParser class. Its intended to be used only when collecting prompt references, nothing else.
 
@@ -444,12 +528,18 @@ https://github.com/lastmile-ai/aiconfig/blob/v1.1.8/extensions/Gemini/python/src
                     elif isinstance(parts, list):
                         return " ".join(parts)
                     else:
-                        raise Exception(f"Cannot get prompt template string from prompt input: {prompt.input}")
+                        raise Exception(
+                            f"Cannot get prompt template string from prompt input: {prompt.input}"
+                        )
                 else:
-                    raise Exception(f"Cannot get prompt template string from prompt input: {prompt.input}")
+                    raise Exception(
+                        f"Cannot get prompt template string from prompt input: {prompt.input}"
+                    )
 
         else:
-            raise Exception(f"Cannot get prompt template string from prompt input: {prompt.input}")
+            raise Exception(
+                f"Cannot get prompt template string from prompt input: {prompt.input}"
+            )
 
 
 def refine_chat_completion_params(model_settings):
@@ -469,7 +559,12 @@ def refine_chat_completion_params(model_settings):
     return completion_data
 
 
-def parameterize_supported_gemini_input_data(part: Any, prompt: Prompt, aiconfig: "AIConfigRuntime", input_params: dict[str, Any]):
+def parameterize_supported_gemini_input_data(
+    part: Any,
+    prompt: Prompt,
+    aiconfig: "AIConfigRuntime",
+    input_params: dict[str, Any],
+):
     """
     Parameterizes the input for the Gemini API based on the type of the input part.
     This function specifically handles string-based types in the context of Gemini API.
@@ -490,21 +585,34 @@ def parameterize_supported_gemini_input_data(part: Any, prompt: Prompt, aiconfig
         return resolve_prompt_string(prompt, input_params, aiconfig, part)
     elif isinstance(part, list):
         # This is expecting a list of strings. If its anything else, this will probably fail.
-        return [parameterize_supported_gemini_input_data(item, prompt, aiconfig, input_params) for item in part]
+        return [
+            parameterize_supported_gemini_input_data(
+                item, prompt, aiconfig, input_params
+            )
+            for item in part
+        ]
     elif isinstance(part, dict):
         # Expect "parts" key to be present in role dict
         if "parts" in part:
             part = copy.deepcopy(part)
-            part["parts"] = parameterize_supported_gemini_input_data(part["parts"], prompt, aiconfig, input_params)
+            part["parts"] = parameterize_supported_gemini_input_data(
+                part["parts"], prompt, aiconfig, input_params
+            )
             return part
         else:
-            raise ValueError(f"Input Dictionary to Gemini Model Parser must contain a 'parts' key. Input provided: {part}")
+            raise ValueError(
+                f"Input Dictionary to Gemini Model Parser must contain a 'parts' key. Input provided: {part}"
+            )
     else:
-        raise ValueError(f"Unable to parameterize part. Unsupported type: {type(part)} with value: {part}")
+        raise ValueError(
+            f"Unable to parameterize part. Unsupported type: {type(part)} with value: {part}"
+        )
 
 
 def contains_prompt_template(prompt: Prompt):
     """
     Check if a prompt's input is a valid string.
     """
-    return isinstance(prompt.input, str) or (hasattr(prompt.input, "data") and isinstance(prompt.input.data, str))
+    return isinstance(prompt.input, str) or (
+        hasattr(prompt.input, "data") and isinstance(prompt.input.data, str)
+    )
